 /*
 * file      : file_lc.c
 * project   : xcfa
 * with      : Gtk-2
 *
 * copyright : (C) 2003 - 2010 by Claude Bulin
 *
 * xcfa - GTK+ implementation of the GNU shell command
 * GNU General Public License
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * OLD ADRESS:
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * NEW ADRESS:
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * 
 */


#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#ifdef ENABLE_NLS
	#include <libintl.h>
	#define _(String) gettext (String)
#endif

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
 

#ifdef ENABLE_STATIC_LIBRARY
	#include "../lib/lib.h"
#endif

#include "global.h"
#include "configuser.h"
#include "conv.h"
#include "prg_init.h"
#include "options.h"
#include "tags.h"
#include "cd_cue.h"
#include "file.h"




// 
// 
gint filelc_get_command_line_remove (void)
{
	gint	pos = 0;
	
	// pos < MAX_ARG_CONV
	for (pos = 0; conv.ArgConv [ pos ] != NULL; pos ++) {
		g_free (conv.ArgConv [ pos ]);
		conv.ArgConv [ pos ] = NULL;
	}
	return (0);
}
// 
// 
gint filelc_get_command_line_nice (void)
{
	gint	pos = filelc_get_command_line_remove ();
	
	conv.ArgConv [ pos++ ] = g_strdup ("nice");
	conv.ArgConv [ pos++ ] = g_strdup ("-n");
	conv.ArgConv [ pos++ ] = g_strdup_printf ("%d", Config.Nice);
	
	// RETOURNE LA PROCHANE POSITION DE pos
	return (3);
}
/*
typedef struct {
	TYPE_CONV	 type_conv;
	gboolean	 With_CommandLineUser;
	gchar		*filesrc;
	gchar		*filedest;
	TAGS		*tags;
	gchar		*cdrom;
	gchar		*num_track;
	// gboolean	 BoolSetBitrate;
	gchar		*PtrStrBitrate;
} PARAM_FILELC;
*/
// 
// 
gint filelc_analyse_line (gchar *entry, gint *pos, TAGS *tags)
{
	gchar     **Larrbuf = NULL;
	gchar      *ptr = entry;
	GString    *gstr = NULL;
	gboolean    bool_tag = FALSE;
	gboolean    bool_open = FALSE;
	gint        i;

	while (ptr && *ptr) {
		if (*ptr == '\"') {
			if (bool_open == TRUE) {
				// New = g_list_append (New, g_strdup (gstr->str));
				conv.ArgConv [ (*pos) ++ ] = g_strdup (gstr->str);
				
				g_string_free (gstr, TRUE);
				gstr = NULL;
				ptr ++;
				bool_open = FALSE;
				continue;
			} else {
				ptr ++;
				bool_open = TRUE;
				continue;
			}
		}

		if (bool_open == TRUE) {
			if (gstr == NULL) gstr = g_string_new (NULL);
			g_string_append_printf (gstr, "%c", *ptr);
			ptr ++;
			continue;
		}

		if (*ptr == ' ') {
			if (gstr != NULL && *gstr->str) {
				// New = g_list_append (New, g_strdup (gstr->str));
				conv.ArgConv [ (*pos) ++ ] = g_strdup (gstr->str);
				
				g_string_free (gstr, TRUE);
				gstr = NULL;
			}
			ptr ++;
			continue;
		}
		else if (*ptr != '%') {
			if (gstr == NULL) gstr = g_string_new (NULL);
			g_string_append_printf (gstr, "%c", *ptr);
			ptr ++;
			continue;
		}

		if (*ptr == '%') {
			ptr ++;
			if ( *ptr == 'b' || *ptr == 'a' || *ptr == 't' || *ptr == 'l' || *ptr == 'N' || *ptr == 'G' || *ptr == 'c' || *ptr == 'd') {
				if (gstr == NULL) gstr = g_string_new (NULL);
			}

			/* OGGENC_WAV_TO_OGG | OGGENC_FLAC_TO_OGG */
			if (*ptr == 'b' && (*(ptr+1) == ' ' || *(ptr+1) == '\0')) {
				// New = g_list_append (New, g_strdup_printf ("%s", optionsOggenc_get_val_bitrate_oggenc ()));
				conv.ArgConv [ (*pos) ++ ] = g_strdup_printf ("%s", optionsOggenc_get_val_bitrate_oggenc ());
				
				ptr ++;
			}
			/* LAME_WAV_TO_MP3 */
			else if (*ptr == 'b' && *(ptr+1) == 'l' && (*(ptr+2) == ' ' || *(ptr+2) == '\0')) {
				Larrbuf = g_strsplit (optionsLame_get_str_val_bitrate_abr_vbr_lame (), " ", 0);
				for (i=0; Larrbuf[i]; i++) {
					// New = g_list_append (New, g_strdup (Larrbuf[i]));
					conv.ArgConv [ (*pos) ++ ] = g_strdup (Larrbuf[i]);
					
				}
				g_strfreev(Larrbuf);
				ptr ++;
				ptr ++;
			} else if (*ptr == 'a' && (*(ptr+1) == ' ' || *(ptr+1) == '\0')) {
				bool_tag = TRUE == Config.BoolArtistTag && NULL != tags && NULL != tags->Artist && '\0' != *tags->Artist;
				g_string_append_printf (gstr, "%s", bool_tag ? tags->Artist : "*");
				ptr ++;
			} else if (*ptr == 't' && (*(ptr+1) == ' ' || *(ptr+1) == '\0')) {
				bool_tag = TRUE == Config.BoolTitleTag && NULL != tags && NULL != tags->Title && '\0' != *tags->Title;
				g_string_append_printf (gstr, "%s", bool_tag ? tags->Title : "*");
				ptr ++;
			} else if (*ptr == 'l' && (*(ptr+1) == ' ' || *(ptr+1) == '\0')) {
				bool_tag = TRUE == Config.BoolAlbumTag && NULL != tags && NULL != tags->Album && '\0' != *tags->Album;
				g_string_append_printf (gstr, "%s", bool_tag ? tags->Album : "Album");
				ptr ++;
			} else if (*ptr == 'N' && (*(ptr+1) == ' ' || *(ptr+1) == '\0')) {
				bool_tag = TRUE == Config.BoolNumerateTag && NULL != tags && NULL != tags->Number && '\0' != *tags->Number;
				g_string_append_printf (gstr, "%s", bool_tag ? tags->Number : "1");
				ptr ++;
			} else if (*ptr == 'G' && (*(ptr+1) == ' ' || *(ptr+1) == '\0')) {
				bool_tag = TRUE == Config.BoolGenreTag && NULL != tags && NULL != tags->Genre && '\0' != *tags->Genre;
				g_string_append_printf (gstr, "%s", bool_tag ? tags->Genre : "100");
				ptr ++;
			} else if (*ptr == 'c' && (*(ptr+1) == ' ' || *(ptr+1) == '\0')) {
				bool_tag = TRUE == Config.BoolCommentTag && NULL != tags && NULL != tags->Comment && '\0' != *tags->Comment;
				g_string_append_printf (gstr, "%s", bool_tag ? tags->Comment : "Comment");
				ptr ++;
			} else if (*ptr == 'd' && (*(ptr+1) == ' ' || *(ptr+1) == '\0')) {
				bool_tag = TRUE == Config.BoolYearTag && NULL != tags && NULL != tags->Year && '\0' != *tags->Year;
				g_string_append_printf (gstr, "%s", bool_tag ? tags->Year : "2007");
				ptr ++;
			}
			else {
				while (ptr && *ptr != ' ') ptr ++;
			}
		}
	}
	if (gstr != NULL && *gstr->str) {
		// New = g_list_append (New, g_strdup (gstr->str));
		conv.ArgConv [ (*pos) ++ ] = g_strdup (gstr->str);
		
		g_string_free (gstr, TRUE);
		gstr = NULL;
	}
	return (*pos);
}
void filelc_analyse_command_line (PARAM_FILELC *param_filelc, gchar *entry)
{
	gint	pos = 0;
	
	if (param_filelc->type_conv == LAME_WAV_TO_MP3) {
		
		pos = filelc_get_command_line_nice ();
		conv.ArgConv [ pos++ ] = g_strdup ("lame");
		pos = filelc_analyse_line (entry, &pos, param_filelc->tags);
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filesrc);
		conv.ArgConv [ pos++ ] = g_strdup ("-o");
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filedest);
		conv.ArgConv [ pos++ ] = NULL;
	}
	else if (param_filelc->type_conv == OGGENC_WAV_TO_OGG || param_filelc->type_conv == OGGENC_FLAC_TO_OGG) {
		
		pos = filelc_get_command_line_nice ();
		conv.ArgConv [ pos++ ] = g_strdup ("oggenc");
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filesrc);
		pos = filelc_analyse_line (entry, &pos, param_filelc->tags);
		conv.ArgConv [ pos++ ] = g_strdup ("-o");
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filedest);
		conv.ArgConv [ pos++ ] = NULL;
	}
	else if (param_filelc->type_conv == FLAC_WAV_TO_FLAC) {
		
		pos = filelc_get_command_line_nice ();
		conv.ArgConv [ pos++ ] = g_strdup ("flac");
		pos = filelc_analyse_line (entry, &pos, param_filelc->tags);
		conv.ArgConv [ pos++ ] = g_strdup ("-f");
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filesrc);
		conv.ArgConv [ pos++ ] = g_strdup ("-o");
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filedest);
		conv.ArgConv [ pos++ ] = NULL;
	}
	else if (param_filelc->type_conv == FAAC_WAV_TO_M4A) {
		
		pos = filelc_get_command_line_nice ();
		conv.ArgConv [ pos++ ] = g_strdup ("faac");
		pos = filelc_analyse_line (entry, &pos, param_filelc->tags);
		conv.ArgConv [ pos++ ] = g_strdup ("-o");
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filedest);
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filesrc);
		conv.ArgConv [ pos++ ] = NULL;
	}
	else if (param_filelc->type_conv == MPPENC_WAV_TO_MPC) {
		
		pos = filelc_get_command_line_nice ();
		conv.ArgConv [ pos++ ] = g_strdup (prginit_get_name (NMR_musepack_tools_mppenc));
		pos = filelc_analyse_line (entry, &pos, param_filelc->tags);
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filesrc);
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filedest);
		conv.ArgConv [ pos++ ] = NULL;
	}
	else if (param_filelc->type_conv == MAC_WAV_TO_APE || param_filelc->type_conv == MAC_APE_TO_WAV) {
		
		pos = filelc_get_command_line_nice ();
		conv.ArgConv [ pos++ ] = g_strdup (prginit_get_name (NMR_mac));
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filesrc);
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filedest);
		conv.ArgConv [ pos++ ] = NULL;
	}
	else if (param_filelc->type_conv == WAVPACK_WAV_TO_WAVPACK) {
		
		pos = filelc_get_command_line_nice ();
		conv.ArgConv [ pos++ ] = g_strdup ("wavpack");
		pos = filelc_analyse_line (entry, &pos, param_filelc->tags);
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filesrc);
		conv.ArgConv [ pos++ ] = g_strdup ("-o");
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filedest);
		conv.ArgConv [ pos++ ] = NULL;
	}
	else if (param_filelc->type_conv == CDPARANOIA_CD_TO_WAV) {
		
		pos = filelc_get_command_line_nice ();
		conv.ArgConv [ pos++ ] = g_strdup ("cdparanoia");
		conv.ArgConv [ pos++ ] = g_strdup ("-e");
		conv.ArgConv [ pos++ ] = g_strdup ("-d");
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->cdrom);
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->num_track);
		pos = filelc_analyse_line (entry, &pos, param_filelc->tags);
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filedest);
		conv.ArgConv [ pos++ ] = NULL;
	}
	else if (param_filelc->type_conv == CDDA2WAV_CD_TO_WAV) {
		
		pos = filelc_get_command_line_nice ();
		conv.ArgConv [ pos++ ] = g_strdup ("cdda2wav");
		conv.ArgConv [ pos++ ] = g_strdup ("-D");
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->cdrom);
		conv.ArgConv [ pos++ ] = g_strdup ("-O");
		conv.ArgConv [ pos++ ] = g_strdup ("wav");
		conv.ArgConv [ pos++ ] = g_strdup ("-t");
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->num_track);
		conv.ArgConv [ pos++ ] = NULL;
	}
}
// 
// 
gboolean filelc_bool_add_line_command (TYPE_CONV p_type_conv, gchar **entry)
{
	GtkWidget *wd = NULL;
	
	switch (p_type_conv) {
	
	case FLAC_WAV_TO_FLAC :		wd = var_options.Adr_entry_flac_flac;		break;
	case LAME_WAV_TO_MP3 :		wd = var_options.Adr_entry_lame_mp3;		break;
	case OGGENC_WAV_TO_OGG :	wd = var_options.Adr_entry_oggenc_ogg;		break;
	case FAAC_WAV_TO_M4A :		wd = var_options.Adr_entry_faac_m4a;		break;
	case MPPENC_WAV_TO_MPC :	wd = var_options.Adr_entry_musepack_mpc;	break;
	case MAC_WAV_TO_APE :		wd = var_options.Adr_entry_mac_ape;		break;
	case WAVPACK_WAV_TO_WAVPACK :	wd = var_options.Adr_entry_wavpack_wv;		break;
	
	case CDPARANOIA_CD_TO_WAV_EXPERT :
	case CDPARANOIA_CD_TO_WAV :
	case CDDA2WAV_CD_TO_WAV :	return (FALSE);
	
	default :			return (FALSE);
	}
	if (wd == NULL) return (FALSE);
	
	*entry = (gchar *)gtk_entry_get_text (GTK_ENTRY(wd));

	return ((entry != NULL && **entry != '\0' && **entry != ' ') ? TRUE : FALSE);
}
// 
// 
void filelc_get_command_line (PARAM_FILELC *param_filelc)
{
	gint	pos = 0;
	gchar	*entry = NULL;
	
	// OPTIONS LIGNE DE SAISIE UTILISATEUR
	if (TRUE == param_filelc->With_CommandLineUser &&
	    TRUE == filelc_bool_add_line_command (param_filelc->type_conv, &entry)) {
		
		filelc_analyse_command_line (param_filelc, entry);
		return;
	}
	param_filelc->With_CommandLineUser = FALSE;
	
	if (param_filelc->type_conv == LAME_WAV_TO_MP3) {
		
		gchar	**Larrbuf = NULL;
		gint	  i;
		
		pos = filelc_get_command_line_nice ();
		conv.ArgConv [ pos++ ] = g_strdup ("lame");
		conv.ArgConv [ pos++ ] = g_strdup ("-h");
		conv.ArgConv [ pos++ ] = g_strdup ("--nohist");
		conv.ArgConv [ pos++ ] = g_strdup ("--noreplaygain");
		
		if (param_filelc->PtrStrBitrate != NULL) {
			Larrbuf = g_strsplit (param_filelc->PtrStrBitrate, " ", 0);
			for (i=0; Larrbuf[i]; i++) {
				conv.ArgConv [ pos++ ] = g_strdup (Larrbuf[i]);
			}
			g_strfreev(Larrbuf);
			g_free (param_filelc->PtrStrBitrate);
			param_filelc->PtrStrBitrate = NULL;
		}
		
		if (Config.BoolArtistTag || Config.BoolTitleTag || Config.BoolAlbumTag ||
		    Config.BoolNumerateTag || Config.BoolGenreTag || Config.BoolYearTag || Config.BoolCommentTag) {
			conv.ArgConv [ pos++ ] = g_strdup ("--add-id3v2");
		}
		
		if (Config.BoolArtistTag && param_filelc->tags && param_filelc->tags->Artist && *param_filelc->tags->Artist) {
			// Ptr = utils_convert_from_utf8 (param_filelc->tags->Artist);
			// if (*Ptr != '\0') {
				conv.ArgConv [ pos++ ] = g_strdup ("--ta");
				conv.ArgConv [ pos++ ] = g_strdup (param_filelc->tags->Artist);
			// }
			// g_free (Ptr);	Ptr = NULL;
		}
		
		if (Config.BoolTitleTag && param_filelc->tags && param_filelc->tags->Title && *param_filelc->tags->Title) {
			// Ptr = utils_convert_from_utf8 (param_filelc->tags->Title);
			// if (*Ptr != '\0') {
				conv.ArgConv [ pos++ ] = g_strdup ("--tt");
				conv.ArgConv [ pos++ ] = g_strdup (param_filelc->tags->Title);
			// }
			// g_free (Ptr);	Ptr = NULL;
		}
		
		if (Config.BoolAlbumTag && param_filelc->tags && param_filelc->tags->Album && *param_filelc->tags->Album) {
			// Ptr = utils_convert_from_utf8 (param_filelc->tags->Album);
			// if (*Ptr != '\0') {
				conv.ArgConv [ pos++ ] = g_strdup ("--tl");
				conv.ArgConv [ pos++ ] = g_strdup (param_filelc->tags->Album);
			// }
			// g_free (Ptr);	Ptr = NULL;
		}
		
		if (Config.BoolNumerateTag && param_filelc->tags && param_filelc->tags->Number && *param_filelc->tags->Number) {
			conv.ArgConv [ pos++ ] = g_strdup ("--tn");
			conv.ArgConv [ pos++ ] = g_strdup (param_filelc->tags->Number);
		}
		
		if (Config.BoolGenreTag && param_filelc->tags && param_filelc->tags->Genre && *param_filelc->tags->Genre) {
			
			gint value = 147;
			
			if (param_filelc->tags->bool_tag_cd == FALSE) {
				value = tags_get_genre_by_value (param_filelc->tags->Genre);
			} else {
				value = param_filelc->tags->IntGenre;
			}
			if (value < 0 || value > 147) value = 147;
			/*New = g_list_append (New, g_strdup ("--tg"));
			New = g_list_append (New, g_strdup_printf ("%d", value));*/
			conv.ArgConv [ pos++ ] = g_strdup ("--tg");
			conv.ArgConv [ pos++ ] = g_strdup_printf ("%d", value);
		}
		
		if (Config.BoolYearTag && param_filelc->tags && param_filelc->tags->Year && *param_filelc->tags->Year) {
			conv.ArgConv [ pos++ ] = g_strdup ("--ty");
			conv.ArgConv [ pos++ ] = g_strdup (param_filelc->tags->Year);
		}
		
		if (Config.BoolCommentTag && param_filelc->tags && param_filelc->tags->Comment && *param_filelc->tags->Comment) {
			// Ptr = utils_convert_from_utf8 (param_filelc->tags->Comment);
			// if (*Ptr != '\0') {
				conv.ArgConv [ pos++ ] = g_strdup ("--tc");
				conv.ArgConv [ pos++ ] = g_strdup (param_filelc->tags->Comment);
			// }
			// g_free (Ptr);	Ptr = NULL;
		}
		
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filesrc);
		conv.ArgConv [ pos++ ] = g_strdup ("-o");
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filedest);
		conv.ArgConv [ pos++ ] = NULL;
	}
	else if (param_filelc->type_conv == OGGENC_WAV_TO_OGG || param_filelc->type_conv == OGGENC_FLAC_TO_OGG) {

		gchar	**Larrbuf = NULL;
		gint	  i;
		
		pos = filelc_get_command_line_nice ();
		conv.ArgConv [ pos++ ] = g_strdup ("oggenc");
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filesrc);
		
		if (param_filelc->PtrStrBitrate != NULL) {
			Larrbuf = g_strsplit (param_filelc->PtrStrBitrate, " ", 0);
			for (i=0; Larrbuf[i]; i++) {
				conv.ArgConv [ pos++ ] = g_strdup (Larrbuf[i]);
			}
			g_strfreev(Larrbuf);
			g_free (param_filelc->PtrStrBitrate);
			param_filelc->PtrStrBitrate = NULL;
		}

		if (Config.BoolArtistTag && param_filelc->tags && param_filelc->tags->Artist && *param_filelc->tags->Artist) {
			conv.ArgConv [ pos++ ] = g_strdup ("-a");
			conv.ArgConv [ pos++ ] = g_strdup (param_filelc->tags->Artist);
		}
		if (Config.BoolTitleTag && param_filelc->tags && param_filelc->tags->Title && *param_filelc->tags->Title) {
			conv.ArgConv [ pos++ ] = g_strdup ("-t");
			conv.ArgConv [ pos++ ] = g_strdup (param_filelc->tags->Title);
		}
		if (Config.BoolAlbumTag && param_filelc->tags && param_filelc->tags->Album && *param_filelc->tags->Album) {
			conv.ArgConv [ pos++ ] = g_strdup ("-l");
			conv.ArgConv [ pos++ ] = g_strdup (param_filelc->tags->Album);
		}
		if (Config.BoolNumerateTag && param_filelc->tags && param_filelc->tags->Number && *param_filelc->tags->Number) {
			conv.ArgConv [ pos++ ] = g_strdup ("-N");
			conv.ArgConv [ pos++ ] = g_strdup (param_filelc->tags->Number);
		}
		if (Config.BoolGenreTag && param_filelc->tags && param_filelc->tags->Genre && *param_filelc->tags->Genre) {
			conv.ArgConv [ pos++ ] = g_strdup ("-G");
			conv.ArgConv [ pos++ ] = g_strdup (param_filelc->tags->Genre);
		}
		if (Config.BoolYearTag && param_filelc->tags && param_filelc->tags->Year && *param_filelc->tags->Year) {
			conv.ArgConv [ pos++ ] = g_strdup ("--date");
			conv.ArgConv [ pos++ ] = g_strdup (param_filelc->tags->Year);
		}
		if (Config.BoolCommentTag && param_filelc->tags && param_filelc->tags->Comment && *param_filelc->tags->Comment) {
			conv.ArgConv [ pos++ ] = g_strdup ("-c");
			conv.ArgConv [ pos++ ] = g_strdup (param_filelc->tags->Comment);
		}

		conv.ArgConv [ pos++ ] = g_strdup ("-o");
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filedest);
		conv.ArgConv [ pos++ ] = NULL;
	}
	else if (param_filelc->type_conv == FLAC_WAV_TO_FLAC || param_filelc->type_conv == FLAC_FLAC_TO_WAV) {
		
		pos = filelc_get_command_line_nice ();
		conv.ArgConv [ pos++ ] = g_strdup ("flac");
		conv.ArgConv [ pos++ ] = g_strdup_printf ("%s", optionsFlac_get_compression_level_flac ());
		
		if (Config.BoolArtistTag && param_filelc->tags && param_filelc->tags->Artist && *param_filelc->tags->Artist) {
			conv.ArgConv [ pos++ ] = g_strdup ("-T");
			conv.ArgConv [ pos++ ] = g_strdup_printf ("artist=%s",param_filelc->tags->Artist);
		}
		if (Config.BoolTitleTag && param_filelc->tags && param_filelc->tags->Title && *param_filelc->tags->Title) {
			conv.ArgConv [ pos++ ] = g_strdup ("-T");
			conv.ArgConv [ pos++ ] = g_strdup_printf ("title=%s", param_filelc->tags->Title);
		}
		if (Config.BoolAlbumTag && param_filelc->tags && param_filelc->tags->Album && *param_filelc->tags->Album) {
			conv.ArgConv [ pos++ ] = g_strdup ("-T");
			conv.ArgConv [ pos++ ] = g_strdup_printf ("album=%s", param_filelc->tags->Album);
		}
		if (Config.BoolNumerateTag && param_filelc->tags && param_filelc->tags->IntNumber > -1) {
			conv.ArgConv [ pos++ ] = g_strdup ("-T");
			conv.ArgConv [ pos++ ] = g_strdup_printf ("tracknumber=%d", param_filelc->tags->IntNumber);
		}
		if (Config.BoolGenreTag && param_filelc->tags && param_filelc->tags->Genre && *param_filelc->tags->Genre) {
			conv.ArgConv [ pos++ ] = g_strdup ("-T");
			conv.ArgConv [ pos++ ] = g_strdup_printf ("genre=%s", param_filelc->tags->Genre);
		}
		if (Config.BoolYearTag && param_filelc->tags && param_filelc->tags->Year && *param_filelc->tags->Year) {
			conv.ArgConv [ pos++ ] = g_strdup ("-T");
			conv.ArgConv [ pos++ ] = g_strdup_printf ("date=%s", param_filelc->tags->Year);
		}
		if (Config.BoolCommentTag && param_filelc->tags && param_filelc->tags->Comment && *param_filelc->tags->Comment) {
			conv.ArgConv [ pos++ ] = g_strdup ("-T");
			conv.ArgConv [ pos++ ] = g_strdup_printf ("Description=%s",param_filelc->tags->Comment);
		}
		
		if (param_filelc->type_conv == FLAC_FLAC_TO_WAV) conv.ArgConv [ pos++ ] = g_strdup ("-d");
		
		conv.ArgConv [ pos++ ] = g_strdup ("-f");
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filesrc);
		conv.ArgConv [ pos++ ] = g_strdup ("-o");
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filedest);
		conv.ArgConv [ pos++ ] = NULL;
	}
	else if (param_filelc->type_conv == FAAC_WAV_TO_M4A) {

		gchar *Ptr = NULL;
		
		pos = filelc_get_command_line_nice ();
		conv.ArgConv [ pos++ ] = g_strdup ("faac");
		
		// filelc_analyse_line (entry, New, param_filelc->tags);
		// -----------------
		
		if ('-' == *OptionsFaac_get_faac_conteneur ()) {
			// New = g_list_append (New, g_strdup_printf ("%s", OptionsFaac_get_faac_conteneur ()));
			conv.ArgConv [ pos++ ] = g_strdup (OptionsFaac_get_faac_conteneur ());
		}
		
		Ptr = OptionsFaac_get_faac_set_choice_vbr_abr ();
		if ( *(Ptr + 1) == 'b') {
			// New = g_list_append (New, g_strdup ("-b"));
			conv.ArgConv [ pos++ ] = g_strdup ("-b");
		}
		else {
			// New = g_list_append (New, g_strdup ("-q"));
			conv.ArgConv [ pos++ ] = g_strdup ("-q");
		}
		// New = g_list_append (New, g_strdup_printf ("%s", (Ptr +3)));
		conv.ArgConv [ pos++ ] = g_strdup_printf ("%s", (Ptr +3));
		
		
		if (Config.BoolArtistTag && param_filelc->tags && param_filelc->tags->Artist && *param_filelc->tags->Artist) {
			// New = g_list_append (New, g_strdup ("--artist"));
			conv.ArgConv [ pos++ ] = g_strdup ("--artist");
			// New = g_list_append (New, g_strdup_printf ("%s", param_filelc->tags->Artist));
			conv.ArgConv [ pos++ ] = g_strdup (param_filelc->tags->Artist);
		}
		if (Config.BoolTitleTag && param_filelc->tags && param_filelc->tags->Title && *param_filelc->tags->Title) {
			// New = g_list_append (New, g_strdup ("--title"));
			conv.ArgConv [ pos++ ] = g_strdup ("--title");
			// New = g_list_append (New, g_strdup_printf ("%s", param_filelc->tags->Title));
			conv.ArgConv [ pos++ ] = g_strdup (param_filelc->tags->Title);
		}
		if (Config.BoolAlbumTag && param_filelc->tags && param_filelc->tags->Album && *param_filelc->tags->Album) {
			// New = g_list_append (New, g_strdup ("--album"));
			conv.ArgConv [ pos++ ] = g_strdup ("--album");
			// New = g_list_append (New, g_strdup_printf ("%s", param_filelc->tags->Album));
			conv.ArgConv [ pos++ ] = g_strdup (param_filelc->tags->Album);
		}
		if (Config.BoolGenreTag && param_filelc->tags && param_filelc->tags->Genre && *param_filelc->tags->Genre) {
			// New = g_list_append (New, g_strdup ("--genre"));
			conv.ArgConv [ pos++ ] = g_strdup ("--genre");
			// New = g_list_append (New, g_strdup_printf ("%s", param_filelc->tags->Genre));
			conv.ArgConv [ pos++ ] = g_strdup (param_filelc->tags->Genre);
		}
		if (Config.BoolYearTag && param_filelc->tags && param_filelc->tags->Year && *param_filelc->tags->Year) {
			// New = g_list_append (New, g_strdup ("--year"));
			conv.ArgConv [ pos++ ] = g_strdup ("--year");
			// New = g_list_append (New, g_strdup_printf ("%s", param_filelc->tags->Year));
			conv.ArgConv [ pos++ ] = g_strdup (param_filelc->tags->Year);
		}
		if (Config.BoolNumerateTag && param_filelc->tags && param_filelc->tags->IntNumber > -1) {
			conv.ArgConv [ pos++ ] = g_strdup ("--track");
			conv.ArgConv [ pos++ ] = g_strdup_printf ("%d", param_filelc->tags->IntNumber);
		}
		if (Config.BoolCommentTag && param_filelc->tags && param_filelc->tags->Comment && *param_filelc->tags->Comment) {
			// New = g_list_append (New, g_strdup ("--comment"));
			conv.ArgConv [ pos++ ] = g_strdup ("--comment");
			// New = g_list_append (New, g_strdup_printf ("%s", param_filelc->tags->Comment));
			conv.ArgConv [ pos++ ] = g_strdup (param_filelc->tags->Comment);
		}

		// -----------------
		
		conv.ArgConv [ pos++ ] = g_strdup ("-o");
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filedest);
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filesrc);
		conv.ArgConv [ pos++ ] = NULL;
	}
	else if (param_filelc->type_conv == MPPENC_WAV_TO_MPC) {
		
		pos = filelc_get_command_line_nice ();
		conv.ArgConv [ pos++ ] = g_strdup ("mppenc");
		conv.ArgConv [ pos++ ] = g_strdup ("--verbose");
		conv.ArgConv [ pos++ ] = g_strdup ("--overwrite");
		conv.ArgConv [ pos++ ] = g_strdup (optionsMusepack_get_quality_mppenc ());

		if (Config.BoolArtistTag && param_filelc->tags && param_filelc->tags->Artist && *param_filelc->tags->Artist) {
			// New = g_list_append (New, g_strdup ("--artist"));
			// Ptr = utils_convert_from_utf8 (param_filelc->tags->Artist);
			// New = g_list_append (New, g_strdup_printf ("%s", Ptr));
			// g_free (Ptr);	Ptr = NULL;
			conv.ArgConv [ pos++ ] = g_strdup ("--artist");
			conv.ArgConv [ pos++ ] = g_strdup (param_filelc->tags->Artist);
		}
		if (Config.BoolTitleTag && param_filelc->tags && param_filelc->tags->Title && *param_filelc->tags->Title) {
			// New = g_list_append (New, g_strdup ("--title"));
			// Ptr = utils_convert_from_utf8 (param_filelc->tags->Title);
			// New = g_list_append (New, g_strdup_printf ("%s", Ptr));
			// g_free (Ptr);	Ptr = NULL;
			conv.ArgConv [ pos++ ] = g_strdup ("--title");
			conv.ArgConv [ pos++ ] = g_strdup (param_filelc->tags->Title);
		}
		if (Config.BoolAlbumTag && param_filelc->tags && param_filelc->tags->Album && *param_filelc->tags->Album) {
			// New = g_list_append (New, g_strdup ("--album"));
			// Ptr = utils_convert_from_utf8 (param_filelc->tags->Album);
			// New = g_list_append (New, g_strdup_printf ("%s", Ptr));
			// g_free (Ptr);	Ptr = NULL;
			conv.ArgConv [ pos++ ] = g_strdup ("--album");
			conv.ArgConv [ pos++ ] = g_strdup (param_filelc->tags->Album);
		}
		if (Config.BoolGenreTag && param_filelc->tags && param_filelc->tags->Genre && *param_filelc->tags->Genre) {
			conv.ArgConv [ pos++ ] = g_strdup ("--genre");
			conv.ArgConv [ pos++ ] = g_strdup (param_filelc->tags->Genre);
		}
		if (Config.BoolYearTag && param_filelc->tags && param_filelc->tags->Year && *param_filelc->tags->Year) {
			conv.ArgConv [ pos++ ] = g_strdup ("--year");
			conv.ArgConv [ pos++ ] = g_strdup (param_filelc->tags->Year);
		}
		if (Config.BoolNumerateTag && param_filelc->tags && param_filelc->tags->Number && *param_filelc->tags->Number) {
			conv.ArgConv [ pos++ ] = g_strdup ("--track");
			conv.ArgConv [ pos++ ] = g_strdup (param_filelc->tags->Number);
		}
		if (Config.BoolCommentTag && param_filelc->tags && param_filelc->tags->Comment && *param_filelc->tags->Comment) {
			// New = g_list_append (New, g_strdup ("--comment"));
			// Ptr = utils_convert_from_utf8 (param_filelc->tags->Comment);
			// New = g_list_append (New, g_strdup_printf ("%s", Ptr));
			// g_free (Ptr);	Ptr = NULL;
			conv.ArgConv [ pos++ ] = g_strdup ("--comment");
			conv.ArgConv [ pos++ ] = g_strdup (param_filelc->tags->Comment);
		}
		
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filesrc);
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filedest);
		conv.ArgConv [ pos++ ] = NULL;
	}
	else if (param_filelc->type_conv == MAC_WAV_TO_APE || param_filelc->type_conv == MAC_APE_TO_WAV) {
		
		pos = filelc_get_command_line_nice ();
		conv.ArgConv [ pos++ ] = g_strdup (prginit_get_name (NMR_mac));
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filesrc);
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filedest);
		
		if (param_filelc->type_conv == MAC_APE_TO_WAV) {
			conv.ArgConv [ pos++ ] = g_strdup ("-d");
		}
		else if (param_filelc->type_conv == MAC_WAV_TO_APE) {
			conv.ArgConv [ pos++ ] = g_strdup (optionsApe_get_compression_level_ape ());
		}
		conv.ArgConv [ pos++ ] = NULL;
	}
	else if (param_filelc->type_conv == WAVPACK_WAV_TO_WAVPACK) {
		
		pos = filelc_get_command_line_nice ();
		conv.ArgConv [ pos++ ] = g_strdup ("wavpack");
		conv.ArgConv [ pos++ ] = g_strdup ("-y");
		
		if (*optionsWavpack_get_wavpack_compression () != '\0')
			conv.ArgConv [ pos++ ] = g_strdup (optionsWavpack_get_wavpack_compression ());
		conv.ArgConv [ pos++ ] = g_strdup (optionsWavpack_get_wavpack_sound ());
		
		if (*optionsWavpac_get_wavpack_hybride () != '\0') {
			conv.ArgConv [ pos++ ] = g_strdup (optionsWavpac_get_wavpack_hybride ());
			if (*optionsWavpack_get_wavpack_correction_file () != '\0') {
				conv.ArgConv [ pos++ ] = g_strdup (optionsWavpack_get_wavpack_correction_file ());
			}
			if (*optionsWavpack_get_wavpack_maximum_compression () != '\0') {
				conv.ArgConv [ pos++ ] = g_strdup (optionsWavpack_get_wavpack_maximum_compression ());
			}
		}
		if (*optionsWavpack_get_wavpack_signature_md5 () != '\0') {
			conv.ArgConv [ pos++ ] = g_strdup (optionsWavpack_get_wavpack_signature_md5 ());
		}
		if (*optionsWavpack_get_wavpack_extra_encoding () != '\0') {
			conv.ArgConv [ pos++ ] = g_strdup (optionsWavpack_get_wavpack_extra_encoding ());
		}

		if (Config.BoolArtistTag && param_filelc->tags && param_filelc->tags->Artist && *param_filelc->tags->Artist) {
			conv.ArgConv [ pos++ ] = g_strdup ("-w");
			conv.ArgConv [ pos++ ] = g_strdup_printf ("Artist=%s", param_filelc->tags->Artist);
		}
		if (Config.BoolTitleTag && param_filelc->tags && param_filelc->tags->Title && *param_filelc->tags->Title) {
			conv.ArgConv [ pos++ ] = g_strdup ("-w");
			conv.ArgConv [ pos++ ] = g_strdup_printf ("Title=%s", param_filelc->tags->Title);
		}
		if (Config.BoolAlbumTag && param_filelc->tags && param_filelc->tags->Album && *param_filelc->tags->Album) {
			conv.ArgConv [ pos++ ] = g_strdup ("-w");
			conv.ArgConv [ pos++ ] = g_strdup_printf ("Album=%s", param_filelc->tags->Album);
		}
		if (Config.BoolGenreTag && param_filelc->tags && param_filelc->tags->Genre && *param_filelc->tags->Genre) {
			conv.ArgConv [ pos++ ] = g_strdup ("-w");
			conv.ArgConv [ pos++ ] = g_strdup_printf ("Genre=%s", param_filelc->tags->Genre);
		}
		if (Config.BoolYearTag && param_filelc->tags && param_filelc->tags->Year && *param_filelc->tags->Year) {
			conv.ArgConv [ pos++ ] = g_strdup ("-w");
			conv.ArgConv [ pos++ ] = g_strdup_printf ("Year=%s", param_filelc->tags->Year);
		}
		if (Config.BoolNumerateTag && param_filelc->tags && param_filelc->tags->Number && *param_filelc->tags->Number) {
			conv.ArgConv [ pos++ ] = g_strdup ("-w");
			conv.ArgConv [ pos++ ] = g_strdup_printf ("Track=%s", param_filelc->tags->Number);
		}
		if (Config.BoolCommentTag && param_filelc->tags &&param_filelc->tags->Comment && *param_filelc->tags->Comment) {
			conv.ArgConv [ pos++ ] = g_strdup ("-w");
			conv.ArgConv [ pos++ ] = g_strdup_printf ("Comment=%s", param_filelc->tags->Comment);
		}
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filesrc);
		conv.ArgConv [ pos++ ] = g_strdup ("-o");
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filedest);
		conv.ArgConv [ pos++ ] = NULL;
	}
	else if (param_filelc->type_conv == CDPARANOIA_CD_TO_WAV) {
		
		pos = filelc_get_command_line_nice ();
		conv.ArgConv [ pos++ ] = g_strdup ("cdparanoia");
		conv.ArgConv [ pos++ ] = g_strdup ("-e");
		conv.ArgConv [ pos++ ] = g_strdup ("-d");
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->cdrom);
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->num_track);
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filedest);
		conv.ArgConv [ pos++ ] = NULL;
	}
	else if (param_filelc->type_conv == CDDA2WAV_CD_TO_WAV) {
		
		// cdda2wav -D /dev/hda -O wav -t 7
		//
		pos = filelc_get_command_line_nice ();
		conv.ArgConv [ pos++ ] = g_strdup ("cdda2wav");
		conv.ArgConv [ pos++ ] = g_strdup ("-D");
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->cdrom);
		conv.ArgConv [ pos++ ] = g_strdup ("-O");
		conv.ArgConv [ pos++ ] = g_strdup ("wav");
		conv.ArgConv [ pos++ ] = g_strdup ("-t");
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->num_track);
		conv.ArgConv [ pos++ ] = NULL;
	}
	else if (param_filelc->type_conv == AACPLUSENC_WAV_TO_AAC) {
		
		pos = filelc_get_command_line_nice ();
		conv.ArgConv [ pos++ ] = g_strdup ("aacplusenc");
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filesrc);
		conv.ArgConv [ pos++ ] = g_strdup (param_filelc->filedest);

		if (param_filelc->tags->bool_tag_cd == TRUE) {
			conv.ArgConv [ pos++ ] = g_strdup_printf ("%d", optionsaacplusenc_get_bitrate_stereo ());
		}
		else {
			if (tagswav_file_is_mono (param_filelc->filesrc) == TRUE) {
				conv.ArgConv [ pos++ ] = g_strdup_printf ("%d", optionsaacplusenc_get_bitrate_mono ());
			} else {
				conv.ArgConv [ pos++ ] = g_strdup_printf ("%d", optionsaacplusenc_get_bitrate_stereo ());
			}
		}
		conv.ArgConv [ pos++ ] = NULL;
	}
}
// 
// 
void filelc_get_command_line_extract_elem (gint Elem, gchar *Dev, gchar *Dest)
{
	gint	pos = 0;
	
	for (pos = 0; conv.ArgExtract [ pos ] != NULL; pos ++) {
		g_free (conv.ArgExtract [ pos ]);
		conv.ArgExtract [ pos ] = NULL;
	}
	pos = 0;
	conv.ArgExtract [ pos++ ] = g_strdup ("nice");
	conv.ArgExtract [ pos++ ] = g_strdup ("-n");
	conv.ArgExtract [ pos++ ] = g_strdup_printf ("%d", Config.Nice);
	conv.ArgExtract [ pos++ ] = g_strdup ("cdparanoia");
	conv.ArgExtract [ pos++ ] = g_strdup_printf ("[.%d]-[.0]", Elem);
	conv.ArgExtract [ pos++ ] = g_strdup ("-e");
	conv.ArgExtract [ pos++ ] = g_strdup ("-d");
	conv.ArgExtract [ pos++ ] = g_strdup (Dev);
	conv.ArgExtract [ pos++ ] = g_strdup ("-O");
	conv.ArgExtract [ pos++ ] = g_strdup ("0");
	conv.ArgExtract [ pos++ ] = g_strdup (Dest);
	conv.ArgExtract [ pos++ ] = NULL;
}
// 
// 
void filelc_get_command_line_extract (PARAM_FILELC *param_filelc)
{
	gint	pos = 0;
	
	for (pos = 0; conv.ArgExtract [ pos ] != NULL; pos ++) {
		g_free (conv.ArgExtract [ pos ]);
		conv.ArgExtract [ pos ] = NULL;
	}
	pos = 0;
	if (param_filelc->type_conv == CDPARANOIA_CD_TO_WAV) {
		conv.ArgExtract [ pos++ ] = g_strdup ("nice");
		conv.ArgExtract [ pos++ ] = g_strdup ("-n");
		conv.ArgExtract [ pos++ ] = g_strdup_printf ("%d", Config.Nice);
		conv.ArgExtract [ pos++ ] = g_strdup ("cdparanoia");
		conv.ArgExtract [ pos++ ] = g_strdup ("-e");
		conv.ArgExtract [ pos++ ] = g_strdup ("-d");
		conv.ArgExtract [ pos++ ] = g_strdup (param_filelc->cdrom);
		conv.ArgExtract [ pos++ ] = g_strdup (param_filelc->num_track);
		conv.ArgExtract [ pos++ ] = g_strdup (param_filelc->filedest);
		conv.ArgExtract [ pos++ ] = NULL;
	}
	else if (param_filelc->type_conv == CDPARANOIA_CD_TO_WAV_EXPERT) {
		
		gint NumTrack = atoi(param_filelc->num_track) -1;
		
		conv.ArgExtract [ pos++ ] = g_strdup ("nice");
		conv.ArgExtract [ pos++ ] = g_strdup ("-n");
		conv.ArgExtract [ pos++ ] = g_strdup_printf ("%d", Config.Nice);
		conv.ArgExtract [ pos++ ] = g_strdup ("cdparanoia");
		conv.ArgExtract [ pos++ ] = g_strdup_printf ("[.%d]-[.%d]",
							BaseIoctl.Datas[ NumTrack ].begin < 44
							? 0
							: BaseIoctl.Datas[ NumTrack ].begin,
							NumTrack == 0
							? (BaseIoctl.Datas[ NumTrack ].length + BaseIoctl.Datas[ NumTrack ].begin) -1
							: BaseIoctl.Datas[ NumTrack ].length -1
							);
		conv.ArgExtract [ pos++ ] = g_strdup ("-e");
		conv.ArgExtract [ pos++ ] = g_strdup ("-d");
		conv.ArgExtract [ pos++ ] = g_strdup (param_filelc->cdrom);
		conv.ArgExtract [ pos++ ] = g_strdup ("-O");
		conv.ArgExtract [ pos++ ] = g_strdup ("0");
		conv.ArgExtract [ pos++ ] = g_strdup (param_filelc->filedest);
		conv.ArgExtract [ pos++ ] = NULL;
	}
	else if (param_filelc->type_conv == CDDA2WAV_CD_TO_WAV) {
		conv.ArgExtract [ pos++ ] = g_strdup ("nice");
		conv.ArgExtract [ pos++ ] = g_strdup ("-n");
		conv.ArgExtract [ pos++ ] = g_strdup_printf ("%d", Config.Nice);
		conv.ArgExtract [ pos++ ] = g_strdup (prginit_get_name (NMR_icedax));
		conv.ArgExtract [ pos++ ] = g_strdup ("-D");
		conv.ArgExtract [ pos++ ] = g_strdup (param_filelc->cdrom);
		conv.ArgExtract [ pos++ ] = g_strdup ("-O");
		conv.ArgExtract [ pos++ ] = g_strdup ("wav");
		conv.ArgExtract [ pos++ ] = g_strdup ("-t");
		conv.ArgExtract [ pos++ ] = g_strdup (param_filelc->num_track);
		conv.ArgExtract [ pos++ ] = g_strdup ("--no-infofile");
		conv.ArgExtract [ pos++ ] = NULL;
	}
}
// 
// 
GList *filelc_remove_glist (GList *New)
{
	GList *list = NULL;
	gchar *ptr = NULL;

	if (New != NULL) {
		list = g_list_first (New);
		while (list) {
			if ((ptr = (gchar*)list->data)) {
				g_free (ptr);
				ptr = list->data = NULL;
			}
			list = g_list_next(list);
		}
		g_list_free (New);
		New = NULL;
	}
	return ((GList *)NULL);
}































